//By L4ggcrafter
import { world, system, ItemStack} from '@minecraft/server'
import {VECTOR} from "./mathUtils.js"
import {shoot} from "./gunFunctions/shootProjectile.js"
import {playEffects} from "./gunFunctions/specialEffects.js"
import {setActionBar} from "./gunFunctions/actionBar.js"
import {updateAmmo} from "./gunFunctions/ammoSystem.js"
import {finishReloading} from "./gunFunctions/reloadSystem.js"
import {setMovement} from "./gunFunctions/movement.js"
import {PlayerEquipment} from "./playerComponents.js"

import {Guns} from "./Guns.js"
import {PlayerGunsData, setUpGunsData} from "./PlayerGunsData.js"

system.runInterval(() => {
    
    let players = world.getAllPlayers()
    for (let player of players) {
        if (!isPlayerAlive (player)) {
            onGunUserDead (player)
            continue
        }
        const item = PlayerEquipment.getItemHold(player)
        gunLoop (player, item)
        finishReloading(player, item)
        setActionBar(player)
        setMovement(player, item)
        

    }
}, 0)
function gunLoop (player, item) {
    if (!PlayerGunsData.hasOwnProperty(player.id)) {
        initializePlayerData(player)
        return
    }
    if (item == undefined) return
    const gunId = item.getDynamicProperty("L4gg-gunId")
    updateGunsCooldown(player.id)
    if (!Guns.hasOwnProperty(item.typeId) && gunId == undefined) return
    initializePlayerGunsData(player, item, gunId)
    const GunId = item.getDynamicProperty("L4gg-gunId") 
    let isNotUsingGun = !PlayerGunsData[player.id].isUsing
    if (isNotUsingGun) return
    const GunHoldData = PlayerGunsData[player.id].guns[gunId]
    if (GunHoldData.currentCooldown > 0) return
    updateAmmo(player, item, GunHoldData)
    system.run (() => { shoot(player, item) })
    playEffects(player, item)
    addCooldown (player.id, item.typeId, GunId)
}
function addCooldown (playerId, itemType, gunId) {
    let gunCoolDown = Guns[itemType].cooldown
        PlayerGunsData[playerId].guns[gunId].currentCooldown = gunCoolDown
}
function updateGunsCooldown(playerId) {
    const GunsData = PlayerGunsData[playerId].guns
    for (let i in GunsData) {
        if (GunsData[i].currentCooldown > 0) {
            GunsData[i].currentCooldown--
            
        }
    }
}
function onGunUserDead (player) {
    if (PlayerGunsData.hasOwnProperty(player.id) && PlayerGunsData[player.id].hasOwnProperty("guns")) {
        PlayerGunsData[player.id].isUsing = false
    }
}
function initializePlayerGunsData(player, item, gunId) {
    if (item == undefined || gunId == undefined) return
    let isInitialized = PlayerGunsData[player.id].guns.hasOwnProperty(gunId)
    if (isInitialized) return
    setUpGunsData(player, item, false)
}
function initializePlayerData(player) {
    const item = PlayerEquipment.getItemHold(player)
    if (item == undefined) return
    const gunId = item.getDynamicProperty("L4gg-gunId")
    if (gunId) {
        setUpGunsData(player, item, false)
    }
}
function setPlayerKnockback () {
    if (system.currentTick % 2 == 0) {
        let players = world.getAllPlayers()
        players.forEach((player) => {
            player.applyKnockback(1, 1, 0.15, 0.1)
        })
    }
}
function isPlayerAlive (player) {
    const healthComp = player.getComponent("health")
    const currentHealth = healthComp.currentValue
    return currentHealth > 0
}